;-------------------------------------------------------------------------------
;hexlib2: See hexlib2.h for function prototypes and details
;-------------------------------------------------------------------------------

	include "doorsos.h"
	include "userlib.h"
	include "hexlib2.h"

	xdef	_ti92plus
	xdef    _ti89
	xdef	_library

	xdef	hexlib2@0000
	xdef	hexlib2@0001
	xdef	hexlib2@0002
	xdef	hexlib2@0003
	xdef	hexlib2@0004
	xdef	hexlib2@0005
	xdef	hexlib2@0006
	xdef	hexlib2@0008

;-------------------------------------------------------------------------------
;prnStr: Calls doorsos::DrawStrXY with the params specified

prnStr		macro				;str,y,x,color
	movem.l	d0-d2/a0-a1,-(a7)	;save registers destroyed by DrawStrXY
	move.w	\4,-(a7)
	pea		\1(PC)
	move.w	\2,-(a7)
	move.w	\3,-(a7)
	jsr		doorsos::DrawStrXY
	lea		10(a7),a7
	movem.l	(a7)+,d0-d2/a0-a1
	endm

;-------------------------------------------------------------------------------
;isHex:
;
;	char '0123456789' ... 'ABCDEF' ... 'abcdef'
;	code  48      57       65  70       97 102

hexlib2@0000:
	cmp.w	#'0',d0					;if digit<'0' it's not an hex digit
	bmi		\notIsHex
	cmp.w	#'f',d0					;if digit>'f' it's not an hex digit
	bgt		\notIsHex
	btst.b	#0,\isHexD-'0'(PC,d0.w)	;Z=!isHexD[d0.w-'0']
	rts

\notIsHex:
	move.b	#0,CCR					;clear the Z flag
	rts

\isHexD:							;one of these values will be loaded to CCR
	dcb.b	58-48,0
	dcb.b	65-58,1
	dcb.b	71-65,0
	dcb.b	97-71,1
	dcb.b	103-97,0

;-------------------------------------------------------------------------------
;isBin:
;
;	char '0' '1'
;	code  48 49

	even
hexlib2@0001:
	cmp.w	#'0',d0					;if digit<'0' it's not a bin digit
	bmi		\notIsBin
	cmp.w	#'1',d0					;if digit>'1' it's not a bin digit
	bgt		\notIsBin
	move.b	#4,CCR					;set the Z flag
	rts

\notIsBin:
	move.b	#0,CCR					;clear the Z flag
	rts

;-------------------------------------------------------------------------------
;hexVal:

	even
hexlib2@0002:
	jsr		hexlib2@0000			;if it's not an hex digit
	bne		\notIsHex
	move.b	\hexV-'0'(PC,d0.w),d0	;return the digit value into d0
	move.b	#0,CCR					;clear the Z flag
	rts

\notIsHex:
	move.b	#4,CCR					;set the Z flag
	rts

\hexV:								;one of these values will be loaded to d0
	dc.b	0,1,2,3,4,5,6,7,8,9
	dcb.b	65-58,0
	dc.b	10,11,12,13,14,15
	dcb.b	97-71,0
	dc.b	10,11,12,13,14,15

;-------------------------------------------------------------------------------
;binVal:

	even
hexlib2@0003:
	jsr		hexlib2@0001			;if it's not a bin digit
	bne		\notIsBin
	sub.b	#'0',d0					;return the digit value into d0
	move.b	#0,CCR					;clear the Z flag
	rts

\notIsBin:
	move.b	#4,CCR					;set the Z flag
	rts

;-------------------------------------------------------------------------------
;inputHex:

	even
hexlib2@0004:

	movem.l	d4-d5/a0,-(a7)		;save registers

	lea		\hexStr(PC),a0		;load hexStr with "|            ",0
	move.b	#'|',(a0)+ 			;                   \--maxLen--/
	move.b	d1,d5
	subq	#1,d5
\loop:
	move.b	#' ',(a0)+
	dbra	d5,\loop
	move.b	0,(a0)

	clr.l	d4					;get user input: d4=hex number
	clr.w	d5					;d5=digit counter
	lea		\hexStr(PC),a0
\input:
	prnStr	\hexStr,d3,d2,#4	;getkey
	jsr		userlib::idle_loop
	cmp.w	#$0108,d0			;if key=ESC goto error
	beq		\error
	cmp.w	#$000D,d0			;if key=RETURN goto end
	beq		\end
	cmp.w	#$0101,d0			;if key=BACKSPACE goto backspace
	beq		\backspace
	jsr		hexlib2@0002		;convert d0 to its hex value
	beq		\input				;if it was not an hex digit goto input
	cmp.b	d5,d1				;if d5=d1 we can't insert: goto input
	beq		\input
	move.b	\hexD(PC,d0.w),0(a0,d5.w)
	asl.l	#4,d4				;multiply d4 by $10
	add.l	d0,d4				;add d0 to d4
	addq.b	#1,d5				;increment d5
	move.b	#'|',0(a0,d5.w)
	move.b	0,1(a0,d5.w)
	bra		\input

\backspace:
	cmp.w	#0,d5				;if d5=0 we can't backspace: goto input
	beq		\input
	asr.l	#4,d4				;else divide d4 by $10
	andi.l	#$0FFFFFFF,d4		;get rid of the sign extension
	move.b	#' ',0(a0,d5.w)
	move.b	0,1(a0,d5.w)
	subq.b	#1,d5				;decrement d5
	move.b	#'|',0(a0,d5.w)
	bra		\input

\end:
	move.l	d4,d0				;load the result into d0
	movem.l	(a7)+,d4-d5/a0		;restore registers
	move.b	#0,CCR				;clear the Z flag indicating exit ok
	rts

\error:
	movem.l	(a7)+,d4-d5/a0		;restore registers
	move.b	#4,CCR				;set the Z flag indicating an error
	rts

\hexStr:
	ds.b	10
\hexD:
	dc.b	"0123456789ABCDEF"

;-------------------------------------------------------------------------------
;inputBin:

	even
hexlib2@0005:

	movem.l	d4-d5/a0,-(a7)		;save registers

	lea		\binStr(PC),a0		;load binStr with "|            ",0
	move.b	#'|',(a0)+ 			;                   \--maxLen--/
	move.b	d1,d5
	subq	#1,d5
\loop:
	move.b	#' ',(a0)+
	dbra	d5,\loop
	move.b	0,(a0)

	clr.l	d4					;get user input: d4=bin number
	clr.w	d5					;d5=digit counter
	lea		\binStr(PC),a0
\input:
	prnStr	\binStr,d3,d2,#4	;getkey
	jsr		userlib::idle_loop
	cmp.w	#$0108,d0			;if key=ESC goto error
	beq		\error
	cmp.w	#$000D,d0			;if key=RETURN goto end
	beq		\end
	cmp.w	#$0101,d0			;if key=BACKSPACE goto backspace
	beq		\backspace
	jsr		hexlib2@0003		;convert d0 to its bin value
	beq		\input				;if it was not a bin digit goto input
	cmp.b	d5,d1				;if d5=d1 we can't insert: goto input
	beq		\input
	move.b	\binD(PC,d0.w),0(a0,d5.w)
	asl.l	#1,d4				;multiply d4 by $2
	add.l	d0,d4				;add d0 to d4
	addq.b	#1,d5				;increment d5
	move.b	#'|',0(a0,d5.w)
	move.b	0,1(a0,d5.w)
	bra		\input

\backspace:
	cmp.w	#0,d5				;if d5=0 we can't backspace: goto input
	beq		\input
	asr.l	#1,d4				;else divide d4 by $2
	bclr.l	#31,d4				;get rid of the sign extension
	move.b	#' ',0(a0,d5.w)
	move.b	0,1(a0,d5.w)
	subq.b	#1,d5				;decrement d5
	move.b	#'|',0(a0,d5.w)
	bra		\input

\end:
	move.l	d4,d0				;load the result into d0
	movem.l	(a7)+,d4-d5/a0		;restore registers
	move.b	#0,CCR				;clear the Z flag indicating exit ok
	rts

\error:
	movem.l	(a7)+,d4-d5/a0		;restore registers
	move.b	#4,CCR				;set the Z flag indicating an error
	rts

\binStr:
	ds.b	34
\binD:
	dc.b	"01"

;-------------------------------------------------------------------------------
;hex2Str:

;Param:		Desc:									Where:	Size:	Range:
;value		value to be converted					d0		l		any
;hexStr		absolute address of destination string	a0		l		any
;strLen		lenght of the string					d1		b		1-8

	even
hexlib2@0006:
	movem.l	d2/a1,-(a7)			;save registers
	lea		\hexD(PC),a1		;make a1 point to hexD
	add.w	d1,a0				;make a0 point to hexStr+strLen
	move.b	#$0,(a0)			;null terminate the string
	subq.w	#$01,d1
\loop:
	move.b	d0,d2				;load hexStr[i] with hexD[d0%16]
	and.w	#$0F,d2
	move.b	0(a1,d2.w),-(a0)
	asr.l	#$04,d0				;divide d0 by 16
	dbra	d1,\loop
	movem.l	(a7)+,d2/a1			;restore registers
	rts

\hexD:	dc.b	"0123456789ABCDEF"

;-------------------------------------------------------------------------------
;prnHex: prints an hexadecimal value.

	even
hexlib2@0008:
	lea		\hexStr(PC),a0
	jsr		hexlib2@0006
	prnStr	\hexStr,d3,d2,d4
	rts

\hexStr:
	ds.b	9

	end