;-------------------------------------------------------------------------------
;hexlib2: Library for working with hex and bin numbers, strings and user input.
;         This library is intended to be a replacement to hexlib, porviding much
;         more functionality.
;
;Version: 0.1
;Author:  Boogerman <prog@boogersoft.com>
;Home:    http://www.boogersoft.com/prog
;
;Notes:   For the latest version please visit http://www.boogersoft.com/prog
;         For bug reports send mail to <prog@boogersoft.com>.
;
;         This is my first library for 68000, so I would appreciate suggestions
;         on how to improve my code, what feature should I add, etc... Thanks!
;
;         I will add more functionality soon!
;-------------------------------------------------------------------------------

hexlib2::isHex		equ hexlib2@0000
hexlib2::isBin		equ hexlib2@0001
hexlib2::hexVal		equ hexlib2@0002
hexlib2::binVal		equ hexlib2@0003
hexlib2::inputHex	equ	hexlib2@0004
hexlib2::inputBin	equ	hexlib2@0005
hexlib2::hex2Str	equ	hexlib2@0006
hexlib2::prnHex		equ	hexlib2@0008

;-----------------------------------FUNCTIONS-----------------------------------
;Functions don't destroy any registers unless stated otherwise.  Parameters  are
;listed in the same order they must be pushed into the stack.
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;isHex: sets the Z flag if d0.w is an hex digit and clears it otherwise.
;
;Param:		Desc:									Where:	Size:	Range:
;digit		digit to test							d0		w		any
;
;example:
;
;	jsr		userlib::idle_loop		;get user input to d0
;	jsr		hexlib2::isHex			;if it's an hex digit goto \hexD
;	beq		\hexD
;	...								;not an hex digit
;
;\hexD:
;	...								;do something

;-------------------------------------------------------------------------------
;isBin: same as isHex

;-------------------------------------------------------------------------------
;hexVal: converts an hex digit in d0 to its hex value. The Z flag is set  if  d0
;        is not an hex digit, in wich case it is left  unmodified,  and  cleared
;        otherwise.
;
;Param:		Desc:									Where:	Size:	Range:
;digit		digit to convert						d0		w		any
;
;
;example:
;
;	jsr		userlib::idle_loop		;get user input to d0
;	jsr		hexlib2::hexVal			;if it's not an hex digit goto \notHex
;	beq		\hexD
;	...								;d0 now has the value of the hex digit
;
;\notHex:
;	...								;do something

;-------------------------------------------------------------------------------
;binVal: same as hexVal

;-------------------------------------------------------------------------------
;inputHex: prompts the user for a maxLen digits hex number at screen coordinates
;          {x,y} and stores it in d0.l. The Z flag is set in case of error  (ie:
;          user presses ESC), and cleared otherwise.
;
;Param:		Desc:									Where:	Size:	Range:
;maxLen		max number of digits of the hex	number	d1		w		{1,8}
;x			column of screen						d2		w		any
;y			row of screen							d3		w		any
;
;example:
;
;	move.w	#4,d1
;	move.w	#5,d2
;	move.w	#10,d3
;	jsr		hexlib2::inputHex
;	beq		\error
;	...								;d0.l contains a value in the range $0-$FFFF
;
;\error
;	...								;the user pressed ESC

;-------------------------------------------------------------------------------
;inputBin: same as inputHex, but with maxLen={1-32)

;-------------------------------------------------------------------------------
;hex2Str: converts a value to a strLen bytes long string.
;
;Param:		Desc:									Where:	Size:	Range:
;value		value to be converted					d0		l		any
;hexStr		absolute address of destination string	a0		l		any
;strLen		lenght of the string					d1		w		1-8
;
;note: this function null terminates the string, so you must allocate strLen+1
;      bytes.
;
;example:
;
;	move.l	#$3F0A,d0				;load the value we want to convert into d0.l
;	lea		\myString(PC),a0		;load the address of the string into a0
;	move.w	#6,d1					;we want a 6 digit string
;	jsr		hexlib2::hex2Str		;call the shot
;	...								;myString now contains "003F0A",0
;
;\myString:
;	ds.b	7						;6 bytes + '\0'

;-------------------------------------------------------------------------------
;prnHex: prints an hexadecimal value.
;
;Param:		Desc:									Where:	Size:	Range:
;value		value to be printed						d0		l		any
;nDigits	number of digits to print				d1		w		1-8
;x			column of screen						d2		w		any
;y			row of screen							d3		w		any
;color		color of text							d4		w		0-4
;
;example:
;
;	move.l	#$3F0A,d0				;load the value we want to print into d0.l
;	move.w	#6,d1					;we want to print 6 digits
;	move.w	#5,d2
;	move.w	#10,d3
;	move.w	#4,d4
;	jsr		hexlib2::prnHex			;The string "003F0A" is printed at {5,10}
